package gethTest

import (
	"crypto/ecdsa"
	"encoding/hex"
	"fmt"
	"os"
	"strconv"
)

type tableStorage struct {
	keys [17][]*ecdsa.PrivateKey

	maxKeysPerBucket int
	keysTotal        int
}

// WriteKeysToFolder exports the stored keys to individual files
func (t *tableStorage) WriteKeysToFolder(dir string) {
	err := os.MkdirAll(dir, os.ModePerm)
	if err != nil {
		fmt.Println("Error creating key folder (", err, ")")
	}
	for i, store := range t.keys {
		if len(store) < 1 {
			continue
		}
		for j, key := range store {
			filename := dir + "/" + strconv.Itoa(i) + "_" + strconv.Itoa(j)
			fd, err := os.Create(filename)
			if err != nil {
				fmt.Println("Error creating key file (", err, ")")
			}

			dst := make([]byte, hex.EncodedLen(len(key.D.Bytes())))
			hex.Encode(dst, key.D.Bytes())
			fd.Write(dst)

			fd.Close()
		}
	}
}

// add adds a private key to the specified bucket
func (t *tableStorage) Add(key *ecdsa.PrivateKey, num int) {
	// check size of bucket
	if len(t.keys[num]) >= t.maxKeysPerBucket {
		return
	}
	// discard duplicates
	for _, k := range t.keys[num] {
		if key == k {
			return
		}
	}

	t.keys[num] = append(t.keys[num], key)
	t.keysTotal = t.keysTotal + 1
}

// printKeys prints out all stored private keys
func (t *tableStorage) PrintKeys() {
	fmt.Println("Printing Keys...")
	for i, store := range t.keys {
		if len(store) < 1 {
			continue
		}
		fmt.Println("Bucket", i)
		for _, key := range store {
			fmt.Println(key.D.Bytes())
		}
	}
}

// KeysTotal returns the total number of stored keys
func (t *tableStorage) KeysTotal() int {
	return t.keysTotal
}

// NewKeyStore creates a new key store
func NewTableStorage(maxKeysPerBucket int) *tableStorage {
	t := new(tableStorage)
	t.maxKeysPerBucket = maxKeysPerBucket
	t.keysTotal = 0
	return t
}
