// run and handle a go-ethereum client. We only re-implement the Ethereum
// overlay protocol in this project; a go-ethereum client can therefore be used
// to handle the TCP stuff
package gethTest

import (
	"crypto/ecdsa"
	"io/ioutil"
	"os/exec"
	"strconv"
)

type gethClient struct {
	binary     string
	datadir    string
	nodekeyhex string
	port       int

	priv *ecdsa.PrivateKey
}

func (g *gethClient) Start() error {
	param1 := "--datadir=" + g.datadir
	param2 := "--nodekeyhex=" + g.nodekeyhex
	param3 := "--port=" + strconv.Itoa(g.port)

	tmuxWindowName := "geth-" + strconv.Itoa(g.port)
	cmd1 := exec.Command("tmux", "kill-window", "-t", tmuxWindowName)
	cmd1.Run()
	cmd2 := exec.Command("tmux", "new-session", "-d", "-s", tmuxWindowName, "-n", "myWindow")
	cmd2.Run()

	tmuxCmd := g.binary + " " + param1 + " " + param2 + " " + param3 + " console Enter\n"
	cmd3 := exec.Command("tmux", "send-keys", "-t", tmuxWindowName, tmuxCmd)
	cmd3.Run()

	return nil
}

func (g *gethClient) Stop() error {
	tmuxWindowName := "geth-" + strconv.Itoa(g.port) + ":myWindow"

	cmd := exec.Command("tmux", "kill-window", "-t", tmuxWindowName)
	cmd.Run()

	return nil
}

func (g *gethClient) ChangeNodeKey(nodekeyhexNew string) error {
	g.nodekeyhex = nodekeyhexNew
	g.Stop()
	g.Start()
	return nil
}

func (g *gethClient) IsConnected() bool {
	fileName := g.datadir + "/connNumber.txt"
	content, err := ioutil.ReadFile(fileName)
	if err != nil {
		return false
	}
	val, _ := strconv.Atoi(string(content[0]))
	if val > 0 {
		return true
	}
	return false
}

func NewGethClient(binary, datadir, nodekeyhex string, port int) *gethClient {
	g := new(gethClient)
	g.binary = binary
	g.datadir = datadir
	g.nodekeyhex = nodekeyhex
	g.port = port

	return g
}
